/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wTimer.h"

using namespace fwk;

Timer::Timer()
{
    reset();
}

/**
 * @brief reset the timer
 */

void Timer::reset()
{
    tStart = Times();
    tElaps = Times();
    isOn = false;
}

/**
 * @brief Start the timer from last "elapsed time"
 */

void Timer::start()
{
    isOn = true;
    tStart.read();
}

/**
 * @brief stop the timer
 */

void Timer::stop()
{
    tElaps = read();
    tStart = Times();
    isOn = false;
}

/**
 * @brief read current time (without stopping the timer)
 */

Times Timer::read() const
{
    if (isOn)
        return tElaps + Times().read() - tStart;
    else
        return tElaps;
}

namespace fwk
{
FWK_API std::ostream &
operator<<(std::ostream &out, Timer const &obj)
{
    out << obj.read();
    return out;
}
} // namespace fwk
